/*
 * UPIWIN - Micro Pi Windowing Framework Kernel
 * Copyright (C) 2019 Amy Bowersox/Erbosoft Metaverse Design Solutions
 *    
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *    
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *-------------------------------------------------------------------------
 */
#ifndef __GFXOBJ_H_INCLUDED
#define __GFXOBJ_H_INCLUDED

#include "wintype.h"

/* type definition for a graphics destructor */
typedef void (*GFX_DTOR)(PVOID obj);

/* A graphics object in memory always starts with this. */
typedef struct tagGFXOBJECT {
  UINT32 sig;                 /* signature so we know what something is */
  UINT32 size;                /* size of total allocated memory in block */
  INT32 refcnt;               /* reference count */
  GFX_DTOR dtor;              /* destructor function called when we're no longer needed */
  struct tagGFXOBJECT *next;  /* support double linked lists of obejcts */
  struct tagGFXOBJECT *prev;
} GFXOBJECT, *PGFXOBJECT;

extern void _Go_init(PGFXOBJECT obj, UINT32 sig, UINT32 size);
extern void Go_unchain(PGFXOBJECT obj);
extern INT32 Go_addref(PGFXOBJECT obj);
extern INT32 Go_release(PGFXOBJECT obj);

#endif /* __GFXOBJ_H_INCLUDED */
